@@ -146,27 +146,28 @@ class Interpreter:
         cnt = 0
         index_as_str = ''
         for ind in vind:
+            i = self.InterpretExpression(ind, scope)
+            index_as_str = f'{index_as_str}[{i.Value}]'
             if val.Type not in [types.VariableTypes.Array, types.VariableTypes.Record, types.VariableTypes.String]:
                 if not(val.Type == types.VariableTypes.Nil and pred != val and pred.Type == types.VariableTypes.Record):
-                    raise exceptions.RuntimeError(f'Type error, index not supported on {val.Type}. ({var.Name}{index_as_str})', ind.Token.Location, self.filename)    
+                    raise exceptions.RuntimeError(f'Index type ({val.Type}) error. ({var.Name}{index_as_str})', ind.Token.Location, self.filename)    
             if ind.ExprType == types.ExprTypes.NewArrayIndex:
                 if val.Type != types.VariableTypes.Array:
-                    raise exceptions.RuntimeError(f'Type error, append not supported on {val.Type}. ({var.Name}{index_as_str})', ind.Token.Location, self.filename)
+                    raise exceptions.RuntimeError(f'Append not supported. ({var.Name}{index_as_str})', ind.Token.Location, self.filename)
                 val.Value.append(value)
                 return
-            i = self.InterpretExpression(ind, scope)
             if val.Type == types.VariableTypes.Record:
                 if i.Value not in val.Value:
                     if cnt == lng -1:
                         val.Value[i.Value] = value
                         return
                     else:
-                        raise exceptions.RuntimeError(f'Index ({i.Value}) not found. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
+                        raise exceptions.RuntimeError(f'Index not found. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
             elif val.Type == types.VariableTypes.Array or val.Type == types.VariableTypes.String:
                 if i.Type != types.VariableTypes.Int:
-                    raise exceptions.RuntimeError(f'Invalid index type ({i.Type}), Int index type required. ({var.Name}).', ind.Token.Location, self.filename)
+                    raise exceptions.RuntimeError(f'Invalid index type ({i.Type}). ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
                 if i.Value < 0 or i.Value >= len(val.Value):
-                    raise exceptions.RuntimeError(f'Index ({i.Value}) is out of range. ({var.Name}{index_as_str})', ind.Token.Location, self.filename)
+                    raise exceptions.RuntimeError(f'Index out of range. ({var.Name}{index_as_str})', ind.Token.Location, self.filename)
             elif val.Type == types.VariableTypes.Nil and pred.Type == types.VariableTypes.Record:
                 if cnt == lng -1:
                     val.Value[i.Value] = value_item.ValueItem().NilValue()
@@ -174,13 +175,11 @@ class Interpreter:
                 else:
                     raise exceptions.RuntimeError(f'Invalid index ({i.Type}) on nil value. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
             pred = val
-            index_as_str = f'{index_as_str}[{i.Value}]'
             val = val.Value[i.Value]
             cnt+=1
-
         if expr.Right is not None and expr.Right.ExprType == types.ExprTypes.NewArrayIndex:
             if val.Type != types.VariableTypes.Array:
-                raise exceptions.RuntimeError(f'Type error, append not supported on {val.Type}', ind.Token.Location, self.filename)    
+                raise exceptions.RuntimeError(f'Append not supported. ({var.Name}{index_as_str}', ind.Token.Location, self.filename)    
             val.Value.append(value)
         else:
             if val.Type == types.VariableTypes.Record:
@@ -239,6 +238,8 @@ class Interpreter:
             return self.InterpretLiteralExpression(expr.Left, scope)
         if expr.ExprType == types.ExprTypes.InterpolationPart:
             return self.InterpretBinaryExpression(expr.Right, scope)
+        if expr.ExprType == types.ExprTypes.NewArrayIndex:
+            return value_item.ValueItem().SetFromMacal(expr.Token, types.VariableTypes.NewArrayIndex, [])
         raise Exception("Expression type evaluation not implemented yet: ", expr.ExprType)
 
 
@@ -349,29 +350,30 @@ class Interpreter:
         while expr.ExprType != types.ExprTypes.VariableIndexStart:
             vind.insert(0, expr.Right)
             expr = expr.Left
-        
+        index_as_str = ''
         for ind in vind:
-            if val.Type not in [types.VariableTypes.Array, types.VariableTypes.Record, types.VariableTypes.String]:
-                raise exceptions.RuntimeError(f'Type error, index not supported on {val.Type}. ({var.Name}).', ind.Token.Location, self.filename)
-            if ind.ExprType == types.ExprTypes.NewArrayIndex:
-                raise exceptions.RuntimeError('Append not supported on right hand side.', ind.Token.Location, self.filename)
             i = self.InterpretExpression(ind, scope)
+            index_as_str = f'{index_as_str}[{i.Value}]'
+            if val.Type not in [types.VariableTypes.Array, types.VariableTypes.Record, types.VariableTypes.String]:
+                raise exceptions.RuntimeError(f'Index type ({val.Type}) not supported. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
+            if ind.ExprType == types.ExprTypes.NewArrayIndex:
+                raise exceptions.RuntimeError('Append not supported. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
             if val.Type == types.VariableTypes.Record:
                 if i.Value not in val.Value:
-                    raise exceptions.RuntimeError(f'Index ({i.Value}) not found. ({var.Name}).', ind.Token.Location, self.filename)
+                    raise exceptions.RuntimeError(f'Index not found. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
             elif val.Type == types.VariableTypes.Array or val.Type == types.VariableTypes.String:
                 if i.Type != types.VariableTypes.Int:
-                    raise exceptions.RuntimeError(f'Invalid index type ({i.Type}), Int index type required. ({var.Name}).', ind.Token.Location, self.filename)
+                    raise exceptions.RuntimeError(f'Invalid index type ({i.Type}), Int index type required. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
                 if i.Value < 0 or i.Value >= len(val.Value):
-                    raise exceptions.RuntimeError(f'Index ({i.Value}) is out of range. ({var.Name}).', ind.Token.Location, self.filename)
+                    raise exceptions.RuntimeError(f'Index out of range. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
             else:
-                raise exceptions.RuntimeError(f'Type error, index not supported on {val.Type}. ({var.Name}).', ind.Token.Location, self.filename)
+                raise exceptions.RuntimeError(f'Type error, index not supported on {val.Type}. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
             if val.Type == types.VariableTypes.String:
                 val = value_item.ValueItem().SetFromMacal(val.Token, val.Type, val.Value[i.Value])
             else:
                 val = val.Value[i.Value]
         if expr.Right is not None and expr.Right.ExprType == types.ExprTypes.NewArrayIndex:
-            raise exceptions.RuntimeError(f'Append not supported. ({var.Name}).', expr.Right.Token.Location, self.filename)
+            raise exceptions.RuntimeError(f'Append not supported. ({var.Name}{index_as_str}).', expr.Right.Token.Location, self.filename)
         return val
 
 
@@ -751,6 +753,8 @@ class Interpreter:
 
     
     def SetIndexedIntoVariableValue(self, var: variable.Variable, idx: ast_expr.Expr, data: value_item.ValueItem, scope: mscope.Scope)-> None:
+        if var.Value.Type not in [types.VariableTypes.Array, types.VariableTypes.Record, types.VariableTypes.String]:
+            raise exceptions.RuntimeError(f'Type error, index not supported on {var.Value.Type}', idx.Token.Location, self.filename)    
         expr = idx
         vind = []
         while expr.ExprType != types.ExprTypes.VariableIndexStart:
@@ -759,24 +763,24 @@ class Interpreter:
             vind.insert(0, expr.Right)
             expr = expr.Left
         val = var.Value
-        if val.Type not in [types.VariableTypes.Array, types.VariableTypes.Record, types.VariableTypes.String]:
-            raise exceptions.RuntimeError(f'Type error, index not supported on {val.Type}', ind.Token.Location, self.filename)    
         finalIndex = None
         finalVal = None
+        index_as_str = ''
         for ind in vind:
-            if val.Type not in [types.VariableTypes.Array, types.VariableTypes.Record, types.VariableTypes.String]:
-                raise exceptions.RuntimeError(f'Type error, index not supported on {val.Type}', ind.Token.Location, self.filename)      
             i = self.InterpretExpression(ind, scope)
+            index_as_str = f'{index_as_str}[{i.Value}]'
+            if val.Type not in [types.VariableTypes.Array, types.VariableTypes.Record, types.VariableTypes.String]:
+                raise exceptions.RuntimeError(f'Type error, index not supported on {val.Type}. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)      
             if val.Type == types.VariableTypes.Record:
                 if i.Value not in val.Value:
-                    raise exceptions.RuntimeError(f'Index ({i.Value}) not found.', ind.Token.Location, self.filename)
+                    raise exceptions.RuntimeError(f'Index ({i.Value}) not found. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
             elif val.Type == types.VariableTypes.Array or val.Type == types.VariableTypes.String:
                 if i.Type != types.VariableTypes.Int:
-                    raise exceptions.RuntimeError(f'Invalid index type ({i.Type}), Int index type required.', ind.Token.Location, self.filename)
+                    raise exceptions.RuntimeError(f'Invalid index type ({i.Type}), Int index type required. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
                 if i.Value < 0 or i.Value >= len(val.Value):
-                    raise exceptions.RuntimeError(f'Index ({i.Value}) is out of range.', ind.Token.Location, self.filename)
+                    raise exceptions.RuntimeError(f'Index ({i.Value}) is out of range. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
             else:
-                raise exceptions.RuntimeError(f'Type error, index not supported on {val.Type}', ind.Token.Location, self.filename)
+                raise exceptions.RuntimeError(f'Type error, index not supported on {val.Type}. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
             finalIndex = i.Value
             finalVal = val
             val = val.Value[i.Value]
@@ -890,7 +894,7 @@ class Interpreter:
             return self.EvaluateDatafieldGrouping(expr, rec, scope)
         if expr.ExprType == types.ExprTypes.Function:
             return self.evaluateDatafieldFunction(expr, rec, scope)
-        raise exceptions.RuntimeError(f'Unknown datafield expression type ({expr.ExprType}).', expr.Token.Location, self.filename)
+        raise exceptions.RuntimeError(f'Invalid datafield expression type ({expr.ExprType}).', expr.Token.Location, self.filename)
 
 
 
@@ -980,36 +984,39 @@ class Interpreter:
 
 
     def EvaluateDatafieldIndexedVariable(self, var: variable.Variable, idx: ast_expr.Expr, rec: dict, scope: mscope.Scope)-> value_item.ValueItem:
+        if var.Value.Type not in [types.VariableTypes.Array, types.VariableTypes.Record, types.VariableTypes.String]:
+            raise exceptions.RuntimeError(f'Type error, index not supported on {var.Value.Type}', ind.Token.Location, self.filename)    
         expr = idx
         vind = []
         while expr.ExprType != types.ExprTypes.VariableIndexStart:
             if expr.Right.ExprType == types.ExprTypes.NewArrayIndex:
-                raise exceptions.RuntimeError('Append not supported on right hand side.', ind.Token.Location, self.filename)
+                raise exceptions.RuntimeError(f'Append not supported. ({var.Name})', ind.Token.Location, self.filename)
             vind.insert(0, expr.Right)
             expr = expr.Left
         val = var.Value
-        if val.Type not in [types.VariableTypes.Array, types.VariableTypes.Record, types.VariableTypes.String]:
-            raise exceptions.RuntimeError(f'Type error, index not supported on {val.Type}', ind.Token.Location, self.filename)    
+        index_as_str = ''
         for ind in vind:
-            if val.Type not in [types.VariableTypes.Array, types.VariableTypes.Record, types.VariableTypes.String]:
-                raise exceptions.RuntimeError(f'Type error, index not supported on {val.Type}', ind.Token.Location, self.filename)
             i = self.InterpretExpression(ind, scope)
+            index_as_str = f'{index_as_str}[{i.Value}]'
+            if val.Type not in [types.VariableTypes.Array, types.VariableTypes.Record, types.VariableTypes.String]:
+                raise exceptions.RuntimeError(f'Index type ({val.Type}) error. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
+            
             if val.Type == types.VariableTypes.Record:
                 if i.Value not in val.Value:
-                    raise exceptions.RuntimeError(f'Index ({i.Value}) not found.', ind.Token.Location, self.filename)
+                    raise exceptions.RuntimeError(f'Index not found. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
             elif val.Type == types.VariableTypes.Array or val.Type == types.VariableTypes.String:
                 if i.Type != types.VariableTypes.Int:
-                    raise exceptions.RuntimeError(f'Invalid index type ({i.Type}), Int index type required.', ind.Token.Location, self.filename)
+                    raise exceptions.RuntimeError(f'Invalid index type ({i.Type}). ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
                 if i.Value < 0 or i.Value >= len(val.Value):
-                    raise exceptions.RuntimeError(f'Index ({i.Value}) is out of range.', ind.Token.Location, self.filename)
+                    raise exceptions.RuntimeError(f'Index out of range. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
             else:
-                raise exceptions.RuntimeError(f'Type error, index not supported on {val.Type}', ind.Token.Location, self.filename)
+                raise exceptions.RuntimeError(f'Index type ({val.Type}) error. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
             if val.Type == types.VariableTypes.String:
                 val = value_item.ValueItem().SetFromMacal(val.Token, val.Type, val.Value[i.Value])
             else:
                 val = val.Value[i.Value]
         if expr.Right is not None and expr.Right.ExprType == types.ExprTypes.NewArrayIndex:
-            raise exceptions.RuntimeError(f'Append not supported on right hand side', ind.Token.Location, self.filename)    
+            raise exceptions.RuntimeError(f'Append not supported. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)    
         return val
 
 
