@@ -34,7 +34,6 @@ class Parser:
         self.source: typing.List[token.LexToken]
         self.filename: str
         self.length = 0
-        self.Debug = False
 
 
 
@@ -126,23 +125,18 @@ class Parser:
             if instruction is not None:
                 instructions.append(instruction)
             #except Exception as ex:
-            #    if self.Debug: print(ex)
-            #    break
-            
+            #    break           
         return instructions
 
 
 
     def Expression(self, index: int, inFnDef: bool = False) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("expr")
-        if self.Debug: print(self.Current(index))
         (index, expr) = self.Boolean(index, inFnDef)
         return (index, expr)
 
 
 
     def Boolean(self, index: int, inFnDef: bool = False) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("bool")
         (index, expr) = self.Equality(index, inFnDef)
         (index, match) = self.Match(index, types.LexTokenTypes.Identifier, ['and', 'or'])
         while match:
@@ -155,20 +149,19 @@ class Parser:
 
 
     def Equality(self, index: int, inFnDef: bool = False) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("eq")
         (index, expr) = self.Comparison(index, inFnDef)
         (index, match) = self.Match(index, types.LexTokenTypes.Operator, ['!=', '=='])
         while match:
             operator = self.Previous(index)
             (index, right) = self.Comparison(index, inFnDef)
             expr = ast_expr.Expr(self.Previous(index)).Binary(expr, operator, right)
+            print("Equality: ", expr)
             (index, match) = self.Match(index, types.LexTokenTypes.Operator, ['!=', '=='])
         return (index, expr)
 
 
     
     def Comparison(self, index: int, inFnDef: bool = False) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("comp")
         (index, expr) = self.Term(index, inFnDef)
         (index, match) = self.Match(index, types.LexTokenTypes.Operator, ['<=', '<', '>', '>='])
         while match:
@@ -181,7 +174,6 @@ class Parser:
 
 
     def Term(self, index: int, inFnDef: bool = False) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("term")
         (index, expr) = self.Factor(index, inFnDef)
         (index, match) = self.Match(index, types.LexTokenTypes.Operator, ['+', '-'])
         while match:
@@ -194,20 +186,18 @@ class Parser:
 
 
     def Factor(self, index: int, inFnDef: bool = False) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("factor")
-        (index, expr) = self.Pm(index, inFnDef)
+        (index, expr) = self.PowerOrModulo(index, inFnDef)
         (index, match) = self.Match(index, types.LexTokenTypes.Operator, ['/', '*'])
         while match:
             operator = self.Previous(index)
-            (index, right) = self.Pm(index, inFnDef)
+            (index, right) = self.PowerOrModulo(index, inFnDef)
             expr = ast_expr.Expr(self.Previous(index)).Binary(expr, operator, right)
             (index, match) = self.Match(index, types.LexTokenTypes.Operator, ['/', '*'])
         return (index, expr)
 
 
 
-    def Pm(self, index: int, inFnDef: bool = False) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("pm")
+    def PowerOrModulo(self, index: int, inFnDef: bool = False) -> typing.Tuple[int, ast.AST]:
         (index, expr) = self.Interpolate(index, inFnDef)
         (index, match) = self.Match(index, types.LexTokenTypes.Operator, ['^', '%'])
         while match:
@@ -219,7 +209,6 @@ class Parser:
 
 
     def Interpolate(self, index: int, inFnDef: bool = False) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("interpolate")
         (index, match) = self.Match(index, types.LexTokenTypes.InterpolationStart, '$')
         if match:
             left = self.Previous(index)
@@ -241,7 +230,6 @@ class Parser:
     
 
     def Unary(self, index: int, inFnDef: bool = False) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("unary")
         (index, match) = self.Match(index, types.LexTokenTypes.Operator, ['!', '+', '-'])
         if not match:
             (index, match) = self.Match(index, types.LexTokenTypes.Identifier, ['not'])
@@ -256,7 +244,6 @@ class Parser:
 
 
     def Grouping(self, index: int, inFnDef: bool = False) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("grouping")
         (index, expr) = self.Expression(index, inFnDef)
         index = self.Consume(index, 'Unbalanced grouping, ")" expected.', types.LexTokenTypes.Punctuation, ')')
         expr = ast_expr.Expr(self.Previous(index)).Grouping(expr)
@@ -265,20 +252,16 @@ class Parser:
 
 
     def Primary(self, index: int, inFnDef: bool = False) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("primary")
         (index, match) = self.Match(index, types.LexTokenTypes.Identifier, ['true', 'false'])
         if match:
-            if self.Debug: print("identifier")
             expr = ast_expr.Expr(self.Previous(index)).Literal(types.VariableTypes.Bool)
             return (index, expr)
         (index, match) = self.Match(index, types.LexTokenTypes.String)
         if match:
-            if self.Debug: print("literal")
             expr = ast_expr.Expr(self.Previous(index)).Literal(types.VariableTypes.String)
             return (index, expr)
         (index, match) = self.Match(index, types.LexTokenTypes.Number)
         if match:
-            if self.Debug: print("number literal")
             literal = self.Previous(index)
             valType = types.VariableTypes.Int
             if '.' in literal.Lexeme:
@@ -287,8 +270,8 @@ class Parser:
             return (index, expr)
         (index, match) = self.Match(index, types.LexTokenTypes.Identifier, 'nil')
         if match:
-            if self.Debug: print("nil")
             expr = ast_expr.Expr(self.Previous(index)).Literal(types.VariableTypes.Nil)
+            print("expr.nil: ", expr)
             return (index, expr)
         (index, match) = self.Match(index, 
                 types.LexTokenTypes.Identifier, 
@@ -305,31 +288,25 @@ class Parser:
                     'function'
                 ])
         if inFnDef is True and match:
-            if self.Debug: print("FnDefArg")
             (index, expr) = self.Argument(index)
             return (index, expr)
         elif match:
-            if self.Debug: print("reserved words")
             tok = self.Previous(index)
             if tok.Lexeme == 'array':
-                if self.Debug: print("array")
                 expr = ast_expr.Expr(tok).Literal(types.VariableTypes.Array)
                 return (index, expr)
             elif tok.Lexeme == 'record':
-                if self.Debug: print("record")
                 expr = ast_expr.Expr(tok).Literal(types.VariableTypes.Record)
                 return (index, expr)
             else:
                 raise exceptions.ParserError(message = f'Invalid use of reserved word. ({tok.Lexeme})', loc=tok.Location, filename=self.filename)       
         (index, match) = self.Match(index, types.LexTokenTypes.Identifier)
         if match:
-            if self.Debug: print("Function Call or Variable")
             (index, expr) = self.FunctionCallOrVariable(index)
             return (index, expr)
         # grouping needs to be the last match, because technically a function call is also a grouping..
         (index, match) = self.Match(index, types.LexTokenTypes.Punctuation, '(')
         if match:
-            if self.Debug: print("PRI_GRP")
             (index, expr) = self.Grouping(index)
             return (index, expr)        
         current = self.Current(index)
@@ -337,25 +314,24 @@ class Parser:
 
 
 
-    def __validateReservedWords(self, tok: token.LexToken):
+    def ValidateReservedWords(self, tok: token.LexToken):
         if tok.Lexeme in ['params', 'variable', 'int', 'bool', 'float', 'function', 'array', 'record', 'nil', 'any', 'string']:
             raise exceptions.ParserError(message = f'Invalid use of reserved word. ({tok.Lexeme})', loc=tok.Location, filename=self.filename)
 
 
 
     def Argument(self, index: int) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("Arg")
         tok = self.Previous(index)
         (index, match) = self.Match(index, types.LexTokenTypes.Identifier)
         if match:
             expr = ast_expr.Expr(tok).Argument(self.Previous(index))
             return (index, expr)
-        self.__validateReservedWords(tok)
+        self.ValidateReservedWords(tok)
         raise exceptions.ParserError(message = f'Function argument definition requires an identifier. ({self.Current(index).Type})', loc=tok.Location, filename=self.filename)
 
 
 
-    def getVariableIndexer(self, index: int) -> typing.Tuple[int, ast.AST]:
+    def GetVariableIndexer(self, index: int) -> typing.Tuple[int, ast.AST]:
         (index, match) = self.Match(index, types.LexTokenTypes.Punctuation, '[')
         expr = ast_expr.Expr(self.Previous(index)).VariableIndexStart()
 
@@ -377,13 +353,11 @@ class Parser:
             if not match:
                 raise exceptions.ParserError(message = 'Unbalanced brackets, expected "]".', loc=self.Previous(index).Location, filename=self.filename)
             (index, match) = self.Match(index, types.LexTokenTypes.Punctuation, '[')
-        #self.Consume(index, 'Unbalanced brackets, Expected "]" .', types.LexTokenTypes.Punctuation, ']')
         return (index, expr)
 
 
 
     def FunctionCallArgumentList(self, index: int, inFnDef: bool = False) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("FunctionCallArgumentList")
         (index, match) = self.Match(index, types.LexTokenTypes.Punctuation, '(')
         if not match:
             raise exceptions.ParserError(message='Missing parenthesis, expected "(".', loc=self.Previous(index).Location,filename = self.filename)
@@ -408,26 +382,21 @@ class Parser:
 
 
     def FunctionCallOrVariable(self, index: int, inFnDef: bool = False) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("FNCOV")
         expr = ast_expr.Expr(self.Previous(index)) # the identifier.
         if self.Current(index).Lexeme == '[':
-            if self.Debug: print("VarIndexer")
-            (index, varIndex) = self.getVariableIndexer(index)
+            (index, varIndex) = self.GetVariableIndexer(index)
             expr.Indexed = True
             expr.Left = varIndex
         if self.Current(index).Lexeme == '(':
-            if self.Debug: print("FNCall")
             expr.ExprType = types.ExprTypes.FunctionCall
             (index, expr.Right) = self.FunctionCallArgumentList(index, inFnDef)
         else:
-            if self.Debug: print("Var")
             expr.ExprType = types.ExprTypes.Variable
         return (index, expr)
 
 
 
     def ParseFunctionDefinition(self, index, identifier: token.LexToken) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("FunctionDefinition")
         instruction = ast_function_definition.FunctionDefinition(identifier)
         (index, instruction.Arguments) = self.FunctionCallArgumentList(index=index, inFnDef = True)
         (index, match) = self.Match(index, types.LexTokenTypes.Identifier, 'external')
@@ -446,7 +415,6 @@ class Parser:
 
 
     def ParseFunctionCall(self, index, identifier: token.LexToken, varIndex: ast_expr.Expr) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("ParseFunctionCall")
         instruction = ast_function_call.FunctionCall(identifier, varIndex)
         (index, expr) = self.FunctionCallArgumentList(index-1) # -1 because parse instruction consumed the opening bracket.
         if expr is None or expr.Left is None:
@@ -458,8 +426,6 @@ class Parser:
     
 
     def ParseInstruction(self, index: int) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print()
-        if self.Debug: print("Instruction")
         (index, match) = self.Match(index, types.LexTokenTypes.Punctuation, '{')
         if match:
             return self.ParseBlock(index)
@@ -469,27 +435,20 @@ class Parser:
             message ='Expected instruction identifier.', 
             type = types.LexTokenTypes.Identifier)
         identifier = self.Previous(index)
-        if self.Debug: print(identifier)
-        if self.Debug: print()
-        if self.Debug: print("pk")
         (index, keyword) = self.ParseKeyword(index, identifier)
         if keyword is not None:
             return (index, keyword)
         varIndex = None       
         if self.Current(index).Lexeme == '[':
-            if self.Debug: print("gvi")
-            (index, varIndex) = self.getVariableIndexer(index)      
+            (index, varIndex) = self.GetVariableIndexer(index)      
         (index, match) = self.Match(index, types.LexTokenTypes.Punctuation, '(')
         if match:
-            if self.Debug: print("pfc")
             return self.ParseFunctionCall(index, identifier, varIndex)
         (index, match) = self.Match(index, types.LexTokenTypes.Operator, ['=', '+=', '-=', '/=', '*='])
         if match:
-            if self.Debug: print("pa")
             return self.ParseAssign(index, identifier, varIndex)
         (index, match) = self.Match(index, types.LexTokenTypes.Operator, '=>')
         if match:
-            if self.Debug: print("pfd")
             return self.ParseFunctionDefinition(index, identifier)
         if varIndex is not None:
             raise exceptions.ParserError(message = 'Invalid variable reference in instruction.', loc=identifier.Location, filename=self.filename)
@@ -498,8 +457,6 @@ class Parser:
 
 
     def ParseKeyword(self, index: int, identifier: types.LexTokenType) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("Parse Keyword")
-        if self.Debug: print(identifier.Lexeme)
         if identifier.Lexeme == 'if':
             return self.ParseIf(index, identifier)
         if identifier.Lexeme == 'foreach':
@@ -525,7 +482,6 @@ class Parser:
 
 
     def ParseBlock(self, index: int) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("Block")
         index = self.Consume(index=index, message='Expected block bracket "{".', type=types.LexTokenTypes.Punctuation, lex='{')
         block = ast_block.Block(self.Previous(index))
         (index, match) = self.Match(index, types.LexTokenTypes.Punctuation, '}')
@@ -540,7 +496,6 @@ class Parser:
 
 
     def ParseAssign(self, index: int, identifier: types.LexTokenType, varIndex: ast_expr.Expr) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("Assign")
         operator = self.Previous(index)
         if operator.Lexeme not in ['=', '+=', '-=','/=', '*=']:
             raise exceptions.ParserError(message = f'Invalid assignment operator: {operator}.', loc=self.Previous(index).Location, filename=self.filename)
@@ -552,7 +507,6 @@ class Parser:
 
 
     def ParseConst(self, index: int, identifier: str) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("Const")
         (index, match) = self.Match(index, types.LexTokenTypes.Identifier)
         if match is False:
             raise exceptions.ParserError(message = f'Constant identifier required, got {self.Current(index).Type}.', loc=self.Current(index).Location, filename=self.filename)
@@ -567,7 +521,6 @@ class Parser:
 
 
     def ParseIf(self, index: int, identifier: types.LexTokenType) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("If")
         instruction = ast_if.If(identifier)        
         (index, instruction.Condition) = self.Expression(index)
         if instruction.Condition is None:
@@ -596,7 +549,6 @@ class Parser:
     
 
     def ParseForeach(self, index: int, identifier: types.LexTokenType) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("Foreach")
         fe = ast_foreach.Foreach(identifier)
         (index, match) = self.Match(index, types.LexTokenTypes.Punctuation, '(')
         if match:
@@ -654,18 +606,10 @@ class Parser:
 
 
     def ParseReturn(self, index: int, identifier: types.LexTokenType) -> typing.Tuple[int, ast.AST]:
-        if self.Debug: print("Return")
         ret = ast_return.Return(identifier)
-        #if self.Debug: print("Match (")
-        #(index, match) = self.Match(index, types.LexTokenTypes.Punctuation, '(')
-        #if match:
-        #    raise exceptions.ParserError(message = 'Return is not a function, expected a variable or a literal value.', loc=self.Previous(index).Location, filename=self.filename)
-        if self.Debug: print("->e")            
         (index, ret.Value) = self.Expression(index)
-        if self.Debug: print("vc")
         if ret.Value is None:
             raise exceptions.ParserError(message = 'Expected a variable or a literal value.', loc=self.Previous(index).Location, filename=self.filename)
-        if self.Debug: print("vcc")            
         index = self.Consume(index, 'Expected ";"', types.LexTokenTypes.Punctuation, ';')
         return (index, ret)
 
