@@ -178,6 +178,10 @@ class ValueItem:
     def __eq__(self, other: ValueItem) -> ValueItem:
         if not isinstance(other, ValueItem):
             raise exceptions.RuntimeError(f'EQ: Other is not a ValueItem ({type(other)}) self: {self} other: {other}', None, None)
+        if self.Type == types.VariableTypes.Nil and self.Value != types.VariableTypes.Nil:
+            raise exceptions.RuntimeError(f'EQ: Self is nil type but not nil value. self: {self} other: {other}', None, None)
+        if other.Type == types.VariableTypes.Nil and other.Value != types.VariableTypes.Nil:
+            raise exceptions.RuntimeError(f'EQ: Other is nil type but not nil value. self: {self} other: {other}', None, None)
         res = ValueItem()
         res.Type = types.VariableTypes.Bool
         res.Token = self.Token.Clone()
@@ -302,6 +306,10 @@ class ValueItem:
     def __ne__(self, other: ValueItem) -> ValueItem:
         if not isinstance(other, ValueItem):
             raise exceptions.RuntimeError(f'NE: Other is not a ValueItem ({type(other)}) self: {self} other: {other}', None, None)
+        if self.Type == types.VariableTypes.Nil and self.Value != types.VariableTypes.Nil:
+            raise exceptions.RuntimeError(f'NE: Self is nil type but not nil value. self: {self} other: {other}', None, None)
+        if other.Type == types.VariableTypes.Nil and other.Value != types.VariableTypes.Nil:
+            raise exceptions.RuntimeError(f'NE: Other is nil type but not nil value. self: {self} other: {other}', None, None)
         res = ValueItem().SetFromMacal(self.Token.Clone(), types.VariableTypes.Bool, self.Value != other.Value)
         res.Token.Lexeme = 'true' if res.Value else 'false'
         res.Token.Type = types.LexTokenTypes.Identifier
